package server

import (
	"context"
	"fmt"
	"github.com/gogf/gf/container/gmap"
	"github.com/gogf/gf/frame/g"
	"github.com/gogf/gf/net/gtcp"
	"github.com/gogf/gf/os/glog"
	gatewayV2 "sparrow-sdk/v2"
	"time"
)

type Server struct {
	closeChan chan struct{}
	srv       *gtcp.Server
	ctx       context.Context
	addr      string
	port      int
	gateWay   *gatewayV2.Gateway
	clients   *gmap.Map
}

func NewServer(ctx context.Context, addr string, port int, gw *gatewayV2.Gateway) *Server {
	return &Server{
		closeChan: make(chan struct{}),
		ctx:       ctx,
		addr:      addr,
		port:      port,
		gateWay:   gw,
		clients:   gmap.New(false),
	}
}

func (s *Server) Start() error {
	glog.Printf("服务端启动[%s:%d]", s.addr, s.port)
	srv := gtcp.NewServer(fmt.Sprintf("%s:%d", s.addr, s.port), s.onClientConnect)
	s.srv = srv
	return s.srv.Run()
}

func (s *Server) Stop() {
	s.clients.Iterator(func(k interface{}, v interface{}) bool {
		client := v.(*Client)
		close(client.closeChan)
		return true
	})
	_ = s.srv.Close()
}

func (s *Server) onClientConnect(conn *gtcp.Conn) {
	client := NewClient(s, conn)
	client.closeHandler = func(id string, c *Client) {
		glog.Debugf("客户端断开:%s", id)
		if id != "" {
			_ = s.gateWay.SubDeviceLogout(g.Cfg().GetString("sparrow.DeviceCode"), id)
			s.clients.Remove(id)
		}
	}
	client.regHandler = func(id string, c *Client) {
		_ = s.gateWay.SubDeviceLogin(g.Cfg().GetString("Sparrow.DeviceCode"), id)
		s.clients.Set(id, c)
	}
	go client.SendLoop()
	time.Sleep(10 * time.Second)
	go client.GetSendByte()
}

func (s *Server) ReportStatus(subId string, data interface{}, cmd string) error {
	return s.gateWay.ReportStatus(subId, cmd, data)
}

func (s *Server) GetClient(subId string) *Client {
	client := s.clients.Get(subId)
	if client != nil {
		return client.(*Client)
	}
	return nil
}
